@@ -1,11 +1,13 @@
+import itertools
+import logging
 import random
 import re
-import itertools
 import ssl
+
 import websocket
 
 from streamlink.plugin import Plugin
-from streamlink.plugin.api import useragents, http
+from streamlink.plugin.api import useragents
 from streamlink.stream import RTMPStream
 
 _url_re = re.compile(r"""
@@ -19,7 +21,7 @@ _url_re = re.compile(r"""
 class VLWebSocket(websocket.WebSocket):
     def __init__(self, **_):
         self.session = _.pop("session")
-        self.logger = self.session.logger.new_module("plugins.vaughnlive.websocket")
+        self.logger = logging.getLogger("streamlink.plugins.vaughnlive.websocket")
         sslopt = _.pop("sslopt", {})
         sslopt["cert_reqs"] = ssl.CERT_NONE
         super(VLWebSocket, self).__init__(sslopt=sslopt, **_)
@@ -38,10 +40,10 @@ class VaughnLive(Plugin):
                                                                                                    range(1, 6))]
     origin = "https://vaughnlive.tv"
     rtmp_server_map = {
-        "594140c69edad": "192.240.105.171:1935",
-        "585c4cab1bef1": "192.240.105.171:1935",
-        "5940d648b3929": "192.240.105.171:1935",
-        "5941854b39bc4": "192.240.105.171:1935"
+        "594140c69edad": "192.240.105.171:2935",
+        "585c4cab1bef1": "192.240.105.171:2935",
+        "5940d648b3929": "192.240.105.171:2935",
+        "5941854b39bc4": "192.240.105.171:2935"
     }
     name_remap = {"#vl": "live", "#btv": "btv", "#pt": "pt", "#igb": "instagib", "#vtv": "vtv"}
     domain_map = {"vaughnlive": "#vl", "breakers": "#btv", "instagib": "#igb", "vapers": "#vtv", "pearltime": "#pt"}
